# -*- coding: UTF-8 -*-
"""
A suite of tests for the iiq_patch module
"""
import __builtin__
import unittest
from collections import namedtuple
from mock import patch, MagicMock, mock_open

from iiqtools import iiq_patch
from iiqtools.utils import versions


class TestDataStructures(unittest.TestCase):
    """A set of test cases for the data structures used by iiq_patch"""

    def test_patch_contents(self):
        """The PatchContents API accepts the expected params"""
        patch_contents = iiq_patch.PatchContents(readme='some readme data',
                                                 meta_ini='the contents of meta.ini',
                                                 patched_files={'/patched/file' : '/original/file/location'})

        self.assertTrue(isinstance(patch_contents, iiq_patch._PatchContents))

    @patch.object(iiq_patch, 'tarfile')
    def test_extract_patch_contents_returns(self, fake_tarfile):
        """iiq_patch.extract_patch_contents returns an instance of PatchContents"""
        fake_log = MagicMock()
        fake_tar = MagicMock()
        fake_tarfile.open.return_value = fake_tar
        patch_contents = iiq_patch.extract_patch_contents('/bogus-patch.tgz', fake_log)

        self.assertTrue(isinstance(patch_contents, iiq_patch._PatchContents))

    @patch.object(iiq_patch, 'tarfile')
    def test_extract_patch_contents(self, fake_tarfile):
        """iiq_patch.extract_patch_contents populates PatchContents when patch file is valid"""
        fake_log = MagicMock()
        fake_tar = MagicMock()
        fake_tar.extractfile.return_value.read.return_value = 'some data'
        fake_tar.getmembers.return_value = [self.fake_item_factory('/some/path', isdir=True),
                                            self.fake_item_factory('meta.ini', isdir=False),
                                            self.fake_item_factory('README.txt', isdir=False),
                                            self.fake_item_factory('/patched/file', isdir=False)]
        fake_tarfile.open.return_value = fake_tar
        patch_contents = iiq_patch.extract_patch_contents('/bogus-patch.tgz', fake_log)

        self.assertTrue(isinstance(patch_contents, iiq_patch._PatchContents))
        self.assertEqual(patch_contents.readme, 'some data')
        self.assertEqual(patch_contents.meta_ini, 'some data')
        self.assertEqual(patch_contents.patched_files, {'/patched/file' : 'some data'})


    def fake_item_factory(self, path, isdir=False):
        """Simplifies making fake items within a tarfile"""
        fake_item = MagicMock()
        fake_item.isdir.return_value = isdir
        fake_item.path = path
        return fake_item


class TestValidators(unittest.TestCase):
    """A suite of test cases for the different validators in iiq_patch"""

    @patch.object(iiq_patch.tarfile, 'is_tarfile')
    def test_check_file_not_tar(self, fake_is_tarfile):
        """iiq_patch.check_file raises argparse.ArgumentTypeError if file isn't a tarfile"""
        fake_is_tarfile.return_value = False

        self.assertRaises(iiq_patch.argparse.ArgumentTypeError, iiq_patch.check_file, 'somefile.txt')

    @patch.object(iiq_patch.tarfile, 'is_tarfile')
    def test_check_file_ioerror(self, fake_is_tarfile):
        """iiq_patch.check_file raises argparse.ArgumentTypeError if the value isn't a real file"""
        fake_is_tarfile.side_effect = IOError(9001, 'testerror', 'somefile')

        expected = 'testerror: somefile'
        try:
            iiq_patch.check_file('somefile')
        except iiq_patch.argparse.ArgumentTypeError as doh:
            result = '%s' % doh
        else:
            result = 'Failed'

        self.assertEqual(expected, result)

    @patch.object(iiq_patch.tarfile, 'is_tarfile')
    def test_check_file_ok(self, fake_is_tarfile):
        """iiq_patch.check_file returns the value if it's valid"""
        fake_is_tarfile.return_value = True

        expected = 'somefile'
        result = iiq_patch.check_file('somefile')

        self.assertEqual(expected, result)

    def test_parse_cli_ok(self):
        """iiq_patch.parse_cli returns argparse.Namespace for valid input"""
        args = iiq_patch.parse_cli(['--show'])
        self.assertTrue(isinstance(args, iiq_patch.argparse.Namespace))

    def test_parse_cli_show(self):
        """iiq_patch.parse_cli the --show arg can accept a value"""
        args = iiq_patch.parse_cli(['--show', 'patch1234'])

        self.assertEqual(args.show, 'patch1234')

    def test_patch_is_valid_ok(self):
        """iiq_patch.patch_is_valid returns True when all checks are OK"""
        fake_logger = MagicMock()
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch1234\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        result = iiq_patch.patch_is_valid(patch_contents, fake_logger)
        self.assertTrue(result)

    def test_patch_is_valid_leading_slash(self):
        """iiq_patch.patch_is_valid returns False if the patch file location starts from root"""
        fake_logger = MagicMock()
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch1234\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\n/some/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        result = iiq_patch.patch_is_valid(patch_contents, fake_logger)
        self.assertFalse(result)

    def test_patch_is_valid_no_files(self):
        """iiq_patch.patch_is_valid returns False if there are no files defined in patch"""
        fake_logger = MagicMock()
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch1234\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]',
                                                 patched_files={'some/file.py' : 'data'})
        result = iiq_patch.patch_is_valid(patch_contents, fake_logger)
        self.assertFalse(result)

    def test_patch_is_valid_missing_min_max_versions(self):
        """iiq_patch.patch_is_valid returns False if minimum and maximum are not defined within the patch"""
        fake_logger = MagicMock()
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch1234\nbug=1234\n[version]\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        result = iiq_patch.patch_is_valid(patch_contents, fake_logger)
        self.assertFalse(result)

    def test_patch_is_valid_missing_name_bug(self):
        """iiq_patch.patch_is_valid returns False if name and bug is not defined"""
        fake_logger = MagicMock()
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        result = iiq_patch.patch_is_valid(patch_contents, fake_logger)
        self.assertFalse(result)

    def test_patch_is_valid_missing_headers(self):
        """iiq_patch.patch_is_valid returns False if any expected header is missing in meta.ini"""
        fake_logger = MagicMock()
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='minimum=4.0\nmaximum=4.1.1\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        result = iiq_patch.patch_is_valid(patch_contents, fake_logger)
        self.assertFalse(result)

    def test_patch_is_valid_bad_contents(self):
        """iiq_patch.patch_is_valid returns False is the supplied PatchContents is missing data"""
        fake_logger = MagicMock()
        patch_contents = iiq_patch.PatchContents(readme='',
                                                 meta_ini='',
                                                 patched_files={})
        result = iiq_patch.patch_is_valid(patch_contents, fake_logger)
        self.assertFalse(result)

    @patch.object(iiq_patch, 'md5_matches')
    def test_source_is_patchable_ok(self, fake_md5_matches):
        """iiq_patch.source_is_patchable returns True when all checks complete successfully"""
        fake_logger = MagicMock()
        fake_md5_matches.return_value = True
        iiq_dir = 'some/dir'
        patch_map = {'insightiq/source/file.py' : 'expectedMD5hash'}

        result = iiq_patch.source_is_patchable(patch_map, iiq_dir, fake_logger)
        expected = True

        self.assertEqual(result, expected)

    @patch.object(iiq_patch, 'md5_matches')
    def test_source_is_patchable_ioerror(self, fake_md5_matches):
        """iiq_patch.source_is_patchable returns False if there's an error while reading the source files"""
        fake_logger = MagicMock()
        fake_md5_matches.side_effect = IOError(9001, 'testerror', 'somefile')
        iiq_dir = 'some/dir'
        patch_map = {'insightiq/source/file.py' : 'expectedMD5hash'}

        result = iiq_patch.source_is_patchable(patch_map, iiq_dir, fake_logger)
        expected = False

        self.assertEqual(result, expected)

    @patch.object(iiq_patch, 'md5_matches')
    def test_source_is_patchable_bad_md5(self, fake_md5_matches):
        """iiq_patch.source_is_patchable returns False if the source file md5 doesn't match the expected md5"""
        fake_logger = MagicMock()
        fake_md5_matches.return_value = False
        iiq_dir = 'some/dir'
        patch_map = {'insightiq/source/file.py' : 'expectedMD5hash'}

        result = iiq_patch.source_is_patchable(patch_map, iiq_dir, fake_logger)
        expected = False

        self.assertEqual(result, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    def test_versions_ok(self, fake_get_iiq_version):
        """iiq_patch.version_ok returns True if the installed version of IIQ is within the min/max of the patch"""
        fake_logger = MagicMock()
        fake_get_iiq_version.return_value = iiq_patch.versions.Version(name='iiq', version='4.1.0.0')
        version_info = {'minimum' : '4.0.0.0', 'maximum' : '4.1.1.3'}

        result = iiq_patch.version_ok(version_info, fake_logger)
        expected = True

        self.assertEqual(result, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    def test_versions_ok_min(self, fake_get_iiq_version):
        """iiq_patch.version_ok returns True the the installed version is equal to the minimum patch version"""
        fake_logger = MagicMock()
        fake_get_iiq_version.return_value = iiq_patch.versions.Version(name='iiq', version='4.1.0.0')
        version_info = {'minimum' : '4.1.0.0', 'maximum' : '4.1.1.3'}

        result = iiq_patch.version_ok(version_info, fake_logger)
        expected = True

        self.assertEqual(result, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    def test_versions_ok_max(self, fake_get_iiq_version):
        """iiq_patch.version_ok returns True the the installed version is equal to the maximum patch version"""
        fake_logger = MagicMock()
        fake_get_iiq_version.return_value = iiq_patch.versions.Version(name='iiq', version='4.1.1.3')
        version_info = {'minimum' : '4.1.0.0', 'maximum' : '4.1.1.3'}

        result = iiq_patch.version_ok(version_info, fake_logger)
        expected = True

        self.assertEqual(result, expected)

    def test_md5_matches_ok(self):
        """iiq_patch.md5_matches returns True when the file content's md5 matches the supplied md5"""
        fake_open = mock_open(read_data='foo')
        fake_logger = MagicMock()
        the_hash = 'acbd18db4cc2f85cedef654fccc4a4d8'

        with patch('iiqtools.iiq_patch.open', fake_open, create=True):
            result = iiq_patch.md5_matches('some/file.py', the_hash, fake_logger)
        expected = True

        self.assertEqual(result, expected)

    def test_md5_matches_false(self):
        """iiq_patch.md5_matches returns False when the file content's md5 does not matches the supplied md5"""
        fake_open = mock_open(read_data='asdfwefwewsd')
        fake_logger = MagicMock()
        the_hash = 'acbd18db4cc2f85cedef654fccc4a4d8'

        with patch('iiqtools.iiq_patch.open', fake_open, create=True):
            result = iiq_patch.md5_matches('some/file.py', the_hash, fake_logger)
        expected = False

        self.assertEqual(result, expected)

    def test_expected_backups_ok(self):
        """iiq_patch.expected_backups returns True if the source file copies found meet the patches expectations"""
        fake_logger = MagicMock()
        found_backups = ['path___to___some_file.py']
        expected_backups = ['path/to/some_file.py']

        result = iiq_patch.expected_backups(found_backups, expected_backups, fake_logger)
        expected = True

        self.assertEqual(result, expected)

    def test_expected_backups_duplicates(self):
        """iiq_patch.expected_backups returns False if duplicate source file copies are found"""
        fake_logger = MagicMock()
        found_backups = ['path__to__some_file.py', 'path__to__some_file.py']
        expected_backups = ['path/to/some_file.py']

        result = iiq_patch.expected_backups(found_backups, expected_backups, fake_logger)
        expected = False

        self.assertEqual(result, expected)

    def test_expected_backups_failure(self):
        """iiq_patch.expected_backups returns False if source file copies are don't meet expectations"""
        fake_logger = MagicMock()
        found_backups = ['some_file.py', '']
        expected_backups = ['path/to/some_file.py']

        result = iiq_patch.expected_backups(found_backups, expected_backups, fake_logger)
        expected = False

        self.assertEqual(result, expected)

    def test_expected_backups_extra_found(self):
        """iiq_patch.expected_backups returns False if extra files are located with the backups"""
        fake_logger = MagicMock()
        found_backups = ['path__to__some_file.py', 'path__to__another_file.py']
        expected_backups = ['path/to/some_file.py']

        result = iiq_patch.expected_backups(found_backups, expected_backups, fake_logger)
        expected = False

        self.assertEqual(result, expected)


    def test_expected_backups_missing_copies(self):
        """iiq_patch.expected_backups returns False if we don't find all expected backups"""
        fake_logger = MagicMock()
        found_backups = ['path__to__some_file.py']
        expected_backups = ['path/to/some_file.py', 'path/to/other_file.py']

        result = iiq_patch.expected_backups(found_backups, expected_backups, fake_logger)
        expected = False

        self.assertEqual(result, expected)


class TestUtils(unittest.TestCase):
    """A suite of test cases for the utility functions within iiq_patch"""

    def test_convert_patch_name_to_source(self):
        """"iiq_patch.convert_patch_name correctly converts a backup file name to the source path"""
        name = 'some___backup_copy.py'
        expected = 'some/backup_copy.py'
        result = iiq_patch.convert_patch_name(name, to='source')

        self.assertEqual(expected, result)

    def test_convert_duner_files_to_backup(self):
        """iiq_patch.convert_patch_name handles double-under files, like __init__.py when converting to backup name"""
        name = 'some/__init__.py'
        expected = 'some_____init__.py'
        result = iiq_patch.convert_patch_name(name, to='backup')

        self.assertEqual(expected, result)

    def test_convert_duner_files_to_source(self):
        """iiq_patch.convert_patch_name handles double-under files, like __init__.py when converting to source name"""
        name = 'some_____init__.py'
        expected = 'some/__init__.py'
        result = iiq_patch.convert_patch_name(name, to='source')

        self.assertEqual(expected, result)

    def test_convert_patch_name_to_backup(self):
        """"iiq_patch.convert_patch_name correctly converts source file path to a backup file name"""
        name = 'some/backup_copy.py'
        expected = 'some___backup_copy.py'
        result = iiq_patch.convert_patch_name(name, to='backup')

        self.assertEqual(expected, result)

    def test_convert_patch_name_value_error(self):
        """iiq_patch.convert_patch_name raises ValueError if "to" param is a bad value"""
        self.assertRaises(ValueError, iiq_patch.convert_patch_name, name='some_file.py', to='derp')

    def test_patch_backups_path(self):
        """iiq_patch.patch_backups_path correctly idenitifes where the backups are stored"""
        # WARNING - if you ever have to modify this test, you are most definitely
        # breaking backwards compatibility with existing patches
        result = iiq_patch.patch_backups_path('some/dir')
        expected = 'some/dir/originals'

        self.assertEqual(expected, result)

    def test_join_path(self):
        """iiq_patch.join_path joins args with a '/'"""
        result = iiq_patch.join_path('some', 'path')
        expected = 'some/path'

        self.assertEqual(result, expected)


class TestCommandHandlers(unittest.TestCase):
    """A suite of test cases for the functions that do most of the work"""

    @patch.object(iiq_patch.os, 'remove')
    @patch.object(iiq_patch.shutil, 'copyfile')
    @patch.object(iiq_patch.os.path, 'isfile')
    def test_restore_originals(self, fake_isfile, fake_copyfile, fake_remove):
        """iiq_patch.restore_originals returns None when procedure encounters no issues"""
        fake_isfile.return_value = True
        result = iiq_patch.restore_originals(iiq_dir='some/dir', patch_dir='some/dir/patches', backup_files=['the__backup__file.py'])
        expected = None

        self.assertEqual(result, expected)

    @patch.object(iiq_patch.shutil, 'copyfile')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_install_patch(self, fake_mkdir, fake_copyfile):
        """iiq_patch.install_patch returns None when no issues are encounted"""
        fake_logger = MagicMock()
        fake_open = mock_open(read_data='foo')
        patch_contents = iiq_patch.PatchContents(readme='readme.txt contents',
                                                 meta_ini='foo',
                                                 patched_files={'/some/patched/files.py' : 'data in patched file'})
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=False,
                                         readme='the readme contents',
                                         specific_patch='patch1234',
                                         all_patches=('patchfoo',))
        with patch('iiqtools.iiq_patch.open', fake_open, create=True):
            result = iiq_patch.install_patch(patch_contents, patch_info, 'my_patch', fake_logger)
        expected = None

        self.assertEqual(result, expected)

    @patch.object(__builtin__, 'print')
    @patch.object(versions, 'get_patch_info')
    def test_handle_show_ok(self, fake_get_patch_info, fake_print):
        """iiq_patch.handle_show returns 0 (zero) upon success"""
        fake_logger = MagicMock()
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=False,
                                         readme='the readme contents',
                                         specific_patch='patch1234',
                                         all_patches=('patchfoo',))
        fake_get_patch_info.return_value = patch_info
        exit_code = iiq_patch.handle_show(specific_patch='', log=fake_logger)
        expected = 0

        self.assertEqual(exit_code, expected)

    @patch.object(__builtin__, 'print')
    @patch.object(versions, 'get_patch_info')
    def test_handle_show_details(self, fake_get_patch_info, fake_print):
        """iiq_patch.handle_show returns 0 (zero) upon success"""
        fake_logger = MagicMock()
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch1234',
                                         all_patches=('patch1234',))
        fake_get_patch_info.return_value = patch_info
        exit_code = iiq_patch.handle_show(specific_patch='patch1234', log=fake_logger)
        expected = 0

        self.assertEqual(exit_code, expected)

    @patch.object(__builtin__, 'print')
    @patch.object(versions, 'get_patch_info')
    def test_handle_show_not_installed(self, fake_get_patch_info, fake_print):
        """iiq_patch.handle_show returns 51 if the specific patch is not installed"""
        fake_logger = MagicMock()
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=False,
                                         readme='the readme contents',
                                         specific_patch='patch1234',
                                         all_patches=('patchfoo',))
        fake_get_patch_info.return_value = patch_info
        exit_code = iiq_patch.handle_show(specific_patch='patch1234', log=fake_logger)
        expected = 50

        self.assertEqual(exit_code, expected)

    @patch.object(__builtin__, 'print')
    @patch.object(versions, 'get_patch_info')
    def test_handle_show_no_readme(self, fake_get_patch_info, fake_print):
        """iiq_patch.handle_show returns 51 if the specific patch is not installed"""
        fake_logger = MagicMock()
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='',
                                         specific_patch='patch1234',
                                         all_patches=('patch1234',))
        fake_get_patch_info.return_value = patch_info
        exit_code = iiq_patch.handle_show(specific_patch='patch1234', log=fake_logger)
        expected = 51

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    @patch.object(iiq_patch, 'source_is_patchable')
    @patch.object(versions, 'get_patch_info')
    @patch.object(iiq_patch, 'install_patch')
    @patch.object(iiq_patch, 'extract_patch_contents')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_handle_install(self, fake_mkdir, fake_extract_patch_contents,
        fake_install_patch, fake_get_patch_info, fake_source_is_patchable, fake_get_iiq_version):
        """iiq_patch.handle_install returns zero if patch is successfully installed"""
        fake_logger = MagicMock()
        fake_iiq_version = iiq_patch.versions.Version(name='insightiq', version='4.1.0.3')
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch9001\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        fake_get_iiq_version.return_value = fake_iiq_version
        fake_extract_patch_contents.return_value = patch_contents
        fake_get_patch_info.return_value = patch_info

        exit_code = iiq_patch.handle_install(patch_path='my-patch.tgz', log=fake_logger)
        expected = 0

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    @patch.object(iiq_patch, 'source_is_patchable')
    @patch.object(versions, 'get_patch_info')
    @patch.object(iiq_patch, 'install_patch')
    @patch.object(iiq_patch, 'extract_patch_contents')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_handle_install_no_iiq(self, fake_mkdir, fake_extract_patch_contents,
        fake_install_patch, fake_get_patch_info, fake_source_is_patchable, fake_get_iiq_version):
        """iiq_patch.handle_install returns 100 if InsightIQ is not installed"""
        fake_logger = MagicMock()
        fake_iiq_version = iiq_patch.versions.Version(name='insightiq', version='4.1.0.3')
        patch_info = versions.PatchInfo(iiq_dir='',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch9001\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        fake_get_iiq_version.return_value = fake_iiq_version
        fake_extract_patch_contents.return_value = patch_contents
        fake_get_patch_info.return_value = patch_info

        exit_code = iiq_patch.handle_install(patch_path='my-patch.tgz', log=fake_logger)
        expected = 100

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    @patch.object(iiq_patch, 'source_is_patchable')
    @patch.object(versions, 'get_patch_info')
    @patch.object(iiq_patch, 'install_patch')
    @patch.object(iiq_patch, 'extract_patch_contents')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_handle_install_permissions(self, fake_mkdir, fake_extract_patch_contents,
        fake_install_patch, fake_get_patch_info, fake_source_is_patchable, fake_get_iiq_version):
        """iiq_patch.handle_install returns 13 if it lacks permissions on the file system"""
        fake_logger = MagicMock()
        fake_iiq_version = iiq_patch.versions.Version(name='insightiq', version='4.1.0.3')
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch9001\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        fake_get_iiq_version.return_value = fake_iiq_version
        fake_extract_patch_contents.return_value = patch_contents
        fake_get_patch_info.return_value = patch_info
        fake_mkdir.side_effect = IOError(13, 'permission denied', '/some/install/location')

        exit_code = iiq_patch.handle_install(patch_path='my-patch.tgz', log=fake_logger)
        expected = 13

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    @patch.object(iiq_patch, 'source_is_patchable')
    @patch.object(versions, 'get_patch_info')
    @patch.object(iiq_patch, 'install_patch')
    @patch.object(iiq_patch, 'extract_patch_contents')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_handle_install_dir_exists(self, fake_mkdir, fake_extract_patch_contents,
        fake_install_patch, fake_get_patch_info, fake_source_is_patchable, fake_get_iiq_version):
        """iiq_patch.handle_install returns 0 even if the patches dir already exits"""
        fake_logger = MagicMock()
        fake_iiq_version = iiq_patch.versions.Version(name='insightiq', version='4.1.0.3')
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch9001\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        fake_get_iiq_version.return_value = fake_iiq_version
        fake_extract_patch_contents.return_value = patch_contents
        fake_get_patch_info.return_value = patch_info
        fake_mkdir.side_effect = IOError(17, 'directory exists', '/some/install/location')

        exit_code = iiq_patch.handle_install(patch_path='my-patch.tgz', log=fake_logger)
        expected = 0

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    @patch.object(iiq_patch, 'source_is_patchable')
    @patch.object(versions, 'get_patch_info')
    @patch.object(iiq_patch, 'install_patch')
    @patch.object(iiq_patch, 'extract_patch_contents')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_handle_install_mkdir_error(self, fake_mkdir, fake_extract_patch_contents,
        fake_install_patch, fake_get_patch_info, fake_source_is_patchable, fake_get_iiq_version):
        """iiq_patch.handle_install returns the IOError code if an unexpected error occurs"""
        fake_logger = MagicMock()
        fake_logger.level = 10
        fake_iiq_version = iiq_patch.versions.Version(name='insightiq', version='4.1.0.3')
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch9001\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        fake_get_iiq_version.return_value = fake_iiq_version
        fake_extract_patch_contents.return_value = patch_contents
        fake_get_patch_info.return_value = patch_info
        fake_mkdir.side_effect = IOError(6, 'directory exists', '/some/install/location')

        exit_code = iiq_patch.handle_install(patch_path='my-patch.tgz', log=fake_logger)
        expected = 6

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    @patch.object(iiq_patch, 'source_is_patchable')
    @patch.object(versions, 'get_patch_info')
    @patch.object(iiq_patch, 'install_patch')
    @patch.object(iiq_patch, 'extract_patch_contents')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_handle_install_bad_patch(self, fake_mkdir, fake_extract_patch_contents,
        fake_install_patch, fake_get_patch_info, fake_source_is_patchable, fake_get_iiq_version):
        """iiq_patch.handle_install returns 101 if patch file is malformed"""
        fake_logger = MagicMock()
        fake_iiq_version = iiq_patch.versions.Version(name='insightiq', version='4.1.0.3')
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        patch_contents = iiq_patch.PatchContents(readme='', # no readme = bad patch
                                                 meta_ini='[info]\nname=patch9001\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        fake_get_iiq_version.return_value = fake_iiq_version
        fake_extract_patch_contents.return_value = patch_contents
        fake_get_patch_info.return_value = patch_info

        exit_code = iiq_patch.handle_install(patch_path='my-patch.tgz', log=fake_logger)
        expected = 101

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    @patch.object(iiq_patch, 'source_is_patchable')
    @patch.object(versions, 'get_patch_info')
    @patch.object(iiq_patch, 'install_patch')
    @patch.object(iiq_patch, 'extract_patch_contents')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_handle_install_already_installed(self, fake_mkdir, fake_extract_patch_contents,
        fake_install_patch, fake_get_patch_info, fake_source_is_patchable, fake_get_iiq_version):
        """iiq_patch.handle_install returns 0 if patch already installed"""
        fake_logger = MagicMock()
        fake_iiq_version = iiq_patch.versions.Version(name='insightiq', version='4.1.0.3')
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        patch_contents = iiq_patch.PatchContents(readme='the readme contents',
                                                 meta_ini='[info]\nname=patch1234\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        fake_get_iiq_version.return_value = fake_iiq_version
        fake_extract_patch_contents.return_value = patch_contents
        fake_get_patch_info.return_value = patch_info

        exit_code = iiq_patch.handle_install(patch_path='my-patch.tgz', log=fake_logger)
        expected = 0

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    @patch.object(iiq_patch, 'source_is_patchable')
    @patch.object(versions, 'get_patch_info')
    @patch.object(iiq_patch, 'install_patch')
    @patch.object(iiq_patch, 'extract_patch_contents')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_handle_install_bad_version(self, fake_mkdir, fake_extract_patch_contents,
        fake_install_patch, fake_get_patch_info, fake_source_is_patchable, fake_get_iiq_version):
        """iiq_patch.handle_install returns 102 if patch doesn't apply to installed version of InsightIQ"""
        fake_logger = MagicMock()
        fake_iiq_version = iiq_patch.versions.Version(name='insightiq', version='3.1.0.3')
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        patch_contents = iiq_patch.PatchContents(readme='the readme contents',
                                                 meta_ini='[info]\nname=patch9001\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        fake_get_iiq_version.return_value = fake_iiq_version
        fake_extract_patch_contents.return_value = patch_contents
        fake_get_patch_info.return_value = patch_info

        exit_code = iiq_patch.handle_install(patch_path='my-patch.tgz', log=fake_logger)
        expected = 102

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    @patch.object(iiq_patch, 'source_is_patchable')
    @patch.object(versions, 'get_patch_info')
    @patch.object(iiq_patch, 'install_patch')
    @patch.object(iiq_patch, 'extract_patch_contents')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_handle_install_unpatchable(self, fake_mkdir, fake_extract_patch_contents,
        fake_install_patch, fake_get_patch_info, fake_source_is_patchable, fake_get_iiq_version):
        """iiq_patch.handle_install returns 103 if source fails patchable test"""
        fake_logger = MagicMock()
        fake_iiq_version = iiq_patch.versions.Version(name='insightiq', version='4.1.0.3')
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch9001\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        fake_get_iiq_version.return_value = fake_iiq_version
        fake_extract_patch_contents.return_value = patch_contents
        fake_get_patch_info.return_value = patch_info
        fake_source_is_patchable.return_value = False

        exit_code = iiq_patch.handle_install(patch_path='my-patch.tgz', log=fake_logger)
        expected = 103

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.versions, 'get_iiq_version')
    @patch.object(iiq_patch, 'source_is_patchable')
    @patch.object(versions, 'get_patch_info')
    @patch.object(iiq_patch, 'install_patch')
    @patch.object(iiq_patch, 'extract_patch_contents')
    @patch.object(iiq_patch.os, 'mkdir')
    def test_handle_install_ioerror(self, fake_mkdir, fake_extract_patch_contents,
        fake_install_patch, fake_get_patch_info, fake_source_is_patchable, fake_get_iiq_version):
        """iiq_patch.handle_install returns 104 if IOError occurs in the middle of installing"""
        fake_logger = MagicMock()
        fake_logger.level = 10
        fake_iiq_version = iiq_patch.versions.Version(name='insightiq', version='4.1.0.3')
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        patch_contents = iiq_patch.PatchContents(readme='The readme contents',
                                                 meta_ini='[info]\nname=patch9001\nbug=1234\n[version]\nminimum=4.0\nmaximum=4.1.1\n[files]\nsome/file.py = themd5checksum',
                                                 patched_files={'some/file.py' : 'data'})
        fake_get_patch_info.return_value = patch_info
        fake_get_iiq_version.return_value = fake_iiq_version
        fake_extract_patch_contents.return_value = patch_contents
        fake_install_patch.side_effect = IOError(9, 'testerror', 'somefile')

        exit_code = iiq_patch.handle_install(patch_path='my-patch.tgz', log=fake_logger)
        expected = 104
        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch, 'restore_originals')
    @patch.object(iiq_patch, 'md5_matches')
    @patch.object(iiq_patch.shutil, 'rmtree')
    @patch.object(iiq_patch, 'expected_backups')
    @patch.object(iiq_patch.os, 'listdir')
    @patch.object(iiq_patch, 'ConfigObj')
    @patch.object(versions, 'get_patch_info')
    def test_handle_uninstall(self, fake_get_patch_info, fake_ConfigObj, fake_listdir,
        fake_expected_backups, fake_rmtree, fake_md5_matches, fake_restore_originals):
        """iiq_patch.handle_uninstall return zero when patch uninstall is successful"""
        fake_logger = MagicMock()
        fake_meta_config = MagicMock()
        fake_meta_config.__getitem__.return_value = {'insightiq/patched_file.py' : 'theMD5hash'}
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch1234',
                                         all_patches=('patch1234',))
        fake_get_patch_info.return_value = patch_info
        fake_ConfigObj.return_value = fake_meta_config

        exit_code = iiq_patch.handle_uninstall(patch_name='patch1234', log=fake_logger)
        expected = 0

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch, 'restore_originals')
    @patch.object(iiq_patch, 'md5_matches')
    @patch.object(iiq_patch.shutil, 'rmtree')
    @patch.object(iiq_patch, 'expected_backups')
    @patch.object(iiq_patch.os, 'listdir')
    @patch.object(iiq_patch, 'ConfigObj')
    @patch.object(versions, 'get_patch_info')
    def test_handle_uninstall_not_installed(self, fake_get_patch_info, fake_ConfigObj, fake_listdir,
        fake_expected_backups, fake_rmtree, fake_md5_matches, fake_restore_originals):
        """iiq_patch.handle_uninstall return 200 when patch isn't even installed"""
        fake_logger = MagicMock()
        fake_meta_config = MagicMock()
        fake_meta_config.__getitem__.return_value = {'insightiq/patched_file.py' : 'theMD5hash'}
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=False,
                                         readme='the readme contents',
                                         specific_patch='patch9001',
                                         all_patches=('patch1234',))
        fake_get_patch_info.return_value = patch_info
        fake_ConfigObj.return_value = fake_meta_config

        exit_code = iiq_patch.handle_uninstall(patch_name='patch1234', log=fake_logger)
        expected = 200

        self.assertEqual(exit_code, expected)


    @patch.object(iiq_patch, 'restore_originals')
    @patch.object(iiq_patch, 'md5_matches')
    @patch.object(iiq_patch.shutil, 'rmtree')
    @patch.object(iiq_patch, 'expected_backups')
    @patch.object(iiq_patch.os, 'listdir')
    @patch.object(iiq_patch, 'ConfigObj')
    @patch.object(versions, 'get_patch_info')
    def test_handle_uninstall_ioerror(self, fake_get_patch_info, fake_ConfigObj, fake_listdir,
        fake_expected_backups, fake_rmtree, fake_md5_matches, fake_restore_originals):
        """iiq_patch.handle_uninstall return 201 when patch uninstall encounters an IOError"""
        fake_logger = MagicMock()
        fake_ConfigObj.side_effect = IOError(9, 'some error', 'some file')
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch1234',
                                         all_patches=('patch1234',))
        fake_get_patch_info.return_value = patch_info

        exit_code = iiq_patch.handle_uninstall(patch_name='patch1234', log=fake_logger)
        expected = 201

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch, 'restore_originals')
    @patch.object(iiq_patch, 'md5_matches')
    @patch.object(iiq_patch.shutil, 'rmtree')
    @patch.object(iiq_patch, 'expected_backups')
    @patch.object(iiq_patch.os, 'listdir')
    @patch.object(iiq_patch, 'ConfigObj')
    @patch.object(versions, 'get_patch_info')
    def test_handle_uninstall_bad_backups(self, fake_get_patch_info, fake_ConfigObj, fake_listdir,
        fake_expected_backups, fake_rmtree, fake_md5_matches, fake_restore_originals):
        """iiq_patch.handle_uninstall return 202 when backup source files test fails"""
        fake_logger = MagicMock()
        fake_meta_config = MagicMock()
        fake_meta_config.__getitem__.return_value = {'insightiq/patched_file.py' : 'theMD5hash'}
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch1234',
                                         all_patches=('patch1234',))
        fake_get_patch_info.return_value = patch_info
        fake_expected_backups.return_value = False
        fake_ConfigObj.return_value = fake_meta_config

        exit_code = iiq_patch.handle_uninstall(patch_name='patch1234', log=fake_logger)
        expected = 202

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch, 'restore_originals')
    @patch.object(iiq_patch, 'md5_matches')
    @patch.object(iiq_patch.shutil, 'rmtree')
    @patch.object(iiq_patch, 'expected_backups')
    @patch.object(iiq_patch.os, 'listdir')
    @patch.object(iiq_patch, 'ConfigObj')
    @patch.object(versions, 'get_patch_info')
    def test_handle_uninstall_bad_md5(self, fake_get_patch_info, fake_ConfigObj, fake_listdir,
        fake_expected_backups, fake_rmtree, fake_md5_matches, fake_restore_originals):
        """iiq_patch.handle_uninstall return 203 when patch backup files have bad md5 values"""
        fake_logger = MagicMock()
        fake_meta_config = MagicMock()
        fake_meta_config.__getitem__.return_value = {'insightiq/patched_file.py' : 'theMD5hash'}
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch1234',
                                         all_patches=('patch1234',))
        fake_get_patch_info.return_value = patch_info
        fake_md5_matches.return_value = False
        fake_ConfigObj.return_value = fake_meta_config

        exit_code = iiq_patch.handle_uninstall(patch_name='patch1234', log=fake_logger)
        expected = 203

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch, 'restore_originals')
    @patch.object(iiq_patch, 'md5_matches')
    @patch.object(iiq_patch.shutil, 'rmtree')
    @patch.object(iiq_patch, 'expected_backups')
    @patch.object(iiq_patch.os, 'listdir')
    @patch.object(iiq_patch, 'ConfigObj')
    @patch.object(versions, 'get_patch_info')
    def test_handle_uninstall_ioerror_restore(self, fake_get_patch_info, fake_ConfigObj, fake_listdir,
        fake_expected_backups, fake_rmtree, fake_md5_matches, fake_restore_originals):
        """iiq_patch.handle_uninstall return the errno of the IOError if one is encountered while restoring originals"""
        fake_logger = MagicMock()
        fake_meta_config = MagicMock()
        fake_meta_config.__getitem__.return_value = {'insightiq/patched_file.py' : 'theMD5hash'}
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch1234',
                                         all_patches=('patch1234',))
        fake_get_patch_info.return_value = patch_info
        fake_ConfigObj.return_value = fake_meta_config
        fake_restore_originals.side_effect = IOError(90, 'some error', 'some file')


        exit_code = iiq_patch.handle_uninstall(patch_name='patch1234', log=fake_logger)
        expected = 90

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch, 'restore_originals')
    @patch.object(iiq_patch, 'md5_matches')
    @patch.object(iiq_patch.shutil, 'rmtree')
    @patch.object(iiq_patch, 'expected_backups')
    @patch.object(iiq_patch.os, 'listdir')
    @patch.object(iiq_patch, 'ConfigObj')
    @patch.object(versions, 'get_patch_info')
    def test_handle_uninstall_rm_failure(self, fake_get_patch_info, fake_ConfigObj, fake_listdir,
        fake_expected_backups, fake_rmtree, fake_md5_matches, fake_restore_originals):
        """iiq_patch.handle_uninstall return the errno encounted if it fails to remove the patch reference"""
        fake_logger = MagicMock()
        fake_meta_config = MagicMock()
        fake_meta_config.__getitem__.return_value = {'insightiq/patched_file.py' : 'theMD5hash'}
        patch_info = versions.PatchInfo(iiq_dir='some/dir',
                                         patches_dir='some/dir/patches',
                                         is_installed=True,
                                         readme='the readme contents',
                                         specific_patch='patch1234',
                                         all_patches=('patch1234',))
        fake_get_patch_info.return_value = patch_info
        fake_ConfigObj.return_value = fake_meta_config
        fake_rmtree.side_effect = IOError(95, 'some error', 'some file')


        exit_code = iiq_patch.handle_uninstall(patch_name='patch1234', log=fake_logger)
        expected = 95

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.shell, 'run_cmd')
    @patch.object(iiq_patch, 'handle_uninstall')
    @patch.object(iiq_patch, 'handle_install')
    @patch.object(iiq_patch, 'handle_show')
    @patch.object(iiq_patch, 'get_logger')
    def test_main_show_ok(self, fake_get_logger, fake_handle_show, fake_handle_install,
    fake_handle_uninstall, fake_run_cmd):
        """iiq_patch.main --show returns zero upon success"""
        fake_handle_show.return_value = 0
        fake_handle_install.return_value = 0
        fake_handle_uninstall.return_value = 0
        fake_get_logger.return_value = MagicMock()

        exit_code = iiq_patch.main(['--show'])
        expected = 0

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch, 'check_file')
    @patch.object(iiq_patch.shell, 'run_cmd')
    @patch.object(iiq_patch, 'handle_uninstall')
    @patch.object(iiq_patch, 'handle_install')
    @patch.object(iiq_patch, 'handle_show')
    @patch.object(iiq_patch, 'get_logger')
    def test_main_install_ok(self, fake_get_logger, fake_handle_show, fake_handle_install,
    fake_handle_uninstall, fake_run_cmd, fake_check_file):
        """iiq_patch.main --install returns zero upon success"""
        fake_handle_show.return_value = 0
        fake_handle_install.return_value = 0
        fake_handle_uninstall.return_value = 0
        fake_get_logger.return_value = MagicMock()
        fake_check_file.return_value = 'mypatch.tgz'

        exit_code = iiq_patch.main(['--install', 'mypatch.tgz'])
        expected = 0

        self.assertEqual(exit_code, expected)

    @patch.object(iiq_patch.shell, 'run_cmd')
    @patch.object(iiq_patch, 'handle_uninstall')
    @patch.object(iiq_patch, 'handle_install')
    @patch.object(iiq_patch, 'handle_show')
    @patch.object(iiq_patch, 'get_logger')
    def test_main_uninstall_ok(self, fake_get_logger, fake_handle_show, fake_handle_install,
    fake_handle_uninstall, fake_run_cmd):
        """iiq_patch.main --install returns zero upon success"""
        fake_handle_show.return_value = 0
        fake_handle_install.return_value = 0
        fake_handle_uninstall.return_value = 0
        fake_get_logger.return_value = MagicMock()

        exit_code = iiq_patch.main(['--uninstall', 'mypatch.tgz'])
        expected = 0

        self.assertEqual(exit_code, expected)


if __name__ == '__main__':
    unittest.main()



# -*- coding: UTF-8 -*-
"""
This module contains the business logic for print the versions of InsightIQ
and IIQTool that's installed.
"""
from __future__ import print_function # so we can mock it away in tests
import argparse

from iiqtools.utils import versions


def parse_cli(the_cli_args):
    """Defines the CLI interface for ``iiq_version``"""
    parser = argparse.ArgumentParser(description='Display the versions of InsightIQ and IIQTools',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    args = parser.parse_args(the_cli_args)
    return args


def main(the_cli_args):
    """Entry point for iiq_version script"""
    # Really is here just to support -h/--help
    parse_cli(the_cli_args)

    iiqtools_version = versions.get_iiqtools_version()
    insightiq_version = versions.get_iiq_version()
    print('InsightIQ: %s' % getattr(insightiq_version, 'version', 'None'))
    print('IIQTools: %s' % getattr(iiqtools_version, 'version', 'None'))



# -*- coding: UTF-8 -*-
"""
This script converts the format of datastore exports (not the CSV exports) from
tar to zip. In InsightIQ 4.1, the format was changed to fix bug 162840, in which
an attempt to import a large datastore export would time out. The only change
to the exported data is the format. So to use an export from an older instance
(before 4.1) all you have to convert the format. In other words, the data is still
the same, it's just a different compression format in InsightIQ 4.1.
"""
import os
import re
import zlib
import struct
import tarfile
import zipfile
import argparse
import binascii

from iiqtools.utils.logger import get_logger
from iiqtools.utils.generic import check_path


class BufferedZipFile(zipfile.ZipFile):
    """A subclass of zipfile.ZipFile that can read from a file-like object and
    stream the contents into a new zip file.
    """

    def writebuffered(self, filename, file_handle):
        """Stream write data to the zip archive

        :param filename: **Required** The name to give the data once added to the zip file
        :type filename: String

        :param file_handle: **Required** The file-like object to read
        :type file_handle: Anything that supports the `read <https://docs.python.org/2/tutorial/inputoutput.html#methods-of-file-objects>`_ method
        """
        zinfo = zipfile.ZipInfo(filename=filename)

        zinfo.file_size = file_size = 0
        zinfo.flag_bits = 0x00
        zinfo.header_offset = self.fp.tell()

        self._writecheck(zinfo)
        self._didModify = True

        zinfo.CRC = CRC = 0
        zinfo.compress_size = compress_size = 0
        self.fp.write(zinfo.FileHeader())
        if zinfo.compress_type == zipfile.ZIP_DEFLATED:
            cmpr = zlib.compressobj(zlib.Z_DEFAULT_COMPRESSION, zlib.DEFLATED, -15)
        else:
            cmpr = None

        while True:
            buf = file_handle.read(1024 * 8)
            if not buf:
                break

            file_size = file_size + len(buf)
            CRC = binascii.crc32(buf, CRC) & 0xffffffff
            if cmpr:
                buf = cmpr.compress(buf)
                compress_size = compress_size + len(buf)

            self.fp.write(buf)

        if cmpr:
            buf = cmpr.flush()
            compress_size = compress_size + len(buf)
            self.fp.write(buf)
            zinfo.compress_size = compress_size
        else:
            zinfo.compress_size = file_size

        zinfo.CRC = CRC
        zinfo.file_size = file_size

        position = self.fp.tell()
        self.fp.seek(zinfo.header_offset + 14, 0)
        self.fp.write(struct.pack("<LLL", zinfo.CRC, zinfo.compress_size, zinfo.file_size))
        self.fp.seek(position, 0)
        self.filelist.append(zinfo)
        self.NameToInfo[zinfo.filename] = zinfo


def check_tar(value):
    """Validate that the supplied tar file is an InsightIQ datastore export file.

    :Raises: argparse.ArgumentTypeError

    :Returns: String

    :param value: **Required** The CLI value to validate
    :type value: String
    """
    regex = re.compile("^insightiq_export_\d{10}.tar.gz$")
    try:
        if not os.path.isfile(value):
            msg = 'value %s does not exist' % value
            raise argparse.ArgumentTypeError(msg)
        elif not tarfile.is_tarfile(value):
            msg = 'value %s is not a tar file'
            raise argparse.ArgumentTypeError(msg)
        elif not regex.search(os.path.basename(value)):
            msg = 'value is not a valid InsightIQ datastore export file'
            raise argparse.ArgumentTypeError(msg)
    except IOError as doh:
        # IOError is generated if we cannot read the tar file; i.e. lack permissions
        raise argparse.ArgumentTypeError(doh)
    else:
        return value


def parse_cli(the_cli_args):
    """Handles parsing the CLI, and gives us --help for (basically) free

    :Returns: argparse.Namespace

    :param cli_args: **Required** The arguments passed to the script
    :type cli_args: List
    """
    parser = argparse.ArgumentParser(description='Convert .tar to .zip for IIQ datastore export files',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('-s', '--source-tar', type=check_tar, required=True,
        help='The source .tar file to convert to .zip')
    parser.add_argument('-o', '--output-dir', type=check_path, default='/home/administrator',
        help='The ')

    args = parser.parse_args(the_cli_args)
    return args


def get_timestamp_from_export(source_tar):
    """Allows us to create the new zip archive with the correct timestamp

    :Returns: String

    :param source_tar: **Required** The tar that's being converted to a zip
    :type source_tar: String
    """
    # In case source_tar is provided as a file path
    source_file = os.path.basename(source_tar)
    # file name convention is insightiq_export_TIMESTAMP.tar.gz
    insightiq = 0
    export = 1
    timestamp = 2
    return source_file.rstrip('.tar.gz').split('_')[timestamp]


def joinname(export_dir, file_name):
    """The tar/zip used by InsightIQ expects the data nested in a directory.
    This function handles absolute and relative paths for file_name.

    :Returns: String

    :param export_dir: **Required** The directory name to nest the file under
    :type export_dir: String

    :param file_name: **Required** The name of the filed nested in the directory
    :type file_name: String
    """
    name = os.path.basename(file_name)
    return '%s/%s' % (export_dir, name)


def main(the_cli_args):
    """Entry point for the iiq_tar_to_zip script"""
    args = parse_cli(the_cli_args)
    log = get_logger(log_path='/dev/null', stream_lvl=10, file_lvl=10)

    log.info('Converting %s to zip format', args.source_tar)
    original_timestamp = get_timestamp_from_export(args.source_tar)
    zip_export_dir = "insightiq_export_%s" % original_timestamp
    zip_export_name = zip_export_dir + '.zip'
    zip_export_path = os.path.join(args.output_dir, zip_export_name)

    try:
        zip_export = BufferedZipFile(zip_export_path, mode='w', allowZip64=True)
    except IOError as doh:
        log.error('Unable to create zip file: %s', doh)
        return 1

    tar_export = tarfile.open(args.source_tar)
    tar_files = tar_export.getmembers()
    log.info('InsightIQ datastore tar export contained %s files', len(tar_files))
    for the_file in tar_files:
        file_handle = tar_export.extractfile(the_file)
        log.info('Converting %s', the_file.name)
        try:
            filename = joinname(zip_export_dir, the_file.name)
            zip_export.writebuffered(filename=filename, file_handle=file_handle)
        except (IOError, OSError) as doh:
            log.error(doh)
            log.error('Deleting zip file')
            os.remove(zip_export_path)
            return doh.errno

    zip_export.close()
    tar_export.close()
    log.info('New zip formatted file saved to %s', zip_export_path)
    return 0



# -*- coding: UTF-8 -*-
"""
This module contains functions that parse stdout of a CLI command into a usable
Python data structure.
"""


def ifconfig_to_dict(ifconfig_output):
    """Parse the output from the command `ifconfig` into a dictionary

    :Returns: Dictionary

    :param ifconfig_output: **Required** The pile of stuff outputted by running `ifconfig`
    :type ifconfig_output: String
    """
    base = {'interfaces' : {}}

    iface_name = None
    for line in ifconfig_output.split('\n'):
        if line and not line.startswith(' '):
            # It's the start of a new interface
            chunks = line.split()

            iface_name = chunks[0].strip()
            iface = base['interfaces'].setdefault(iface_name, {})
            iface['link'] = chunks[2].strip()
            iface['hwaddr'] = None
            iface['ipv4'] = {}
            iface['ipv6'] = {}
            iface['flags'] = None,
            iface['mtu'] = None,
            iface['metric'] = None
            iface['rx packets'] = {}
            iface['tx packets'] = {}
            iface['collisions'] = None,
            iface['txqueuelen'] = None,
            iface['rx bytes'] = {}
            iface['tx bytes'] = {}
            if 'hwaddr' in chunks[3].lower():
                iface['hwaddr'] = chunks[4]
            else:
                iface['hwaddr'] = 'Loopback'

        elif line.startswith(' ') and iface_name:
            # keep parsing the interface info
            chunks = line.split()
            if chunks[0] == 'inet':
                ip = chunks[1].split(':')[1]
                bcast = chunks[2].split(':')[1]
                mask = chunks[-1].split(':')[1]
                if bcast == mask:
                    # it's loopback
                    base['interfaces'][iface_name]['ipv4'][ip] = {'mask': mask}
                else:
                    base['interfaces'][iface_name]['ipv4'][ip] = {'bcast': bcast, 'mask': mask}
                continue

            elif chunks[0] == 'inet6':
                ip6 = chunks[2]
                scope = chunks[3].split(':')[1]
                base['interfaces'][iface_name]['ipv6'][ip6] = {'scope' : scope}
                continue

            elif 'Metric' in chunks[-1]:
                metric = int(chunks.pop(-1).split(':')[1])
                mtu = int(chunks.pop(-1).split(':')[1])
                flags = ' '.join(chunks)
                base['interfaces'][iface_name]['mtu'] = mtu
                base['interfaces'][iface_name]['metric'] = metric
                base['interfaces'][iface_name]['flags'] = flags
                continue

            elif 'RX' == chunks[0] and 'packets' in chunks[1]:
                packets = int(chunks[1].split(':')[1])
                errors = int(chunks[2].split(':')[1])
                dropped = int(chunks[3].split(':')[1])
                overruns = int(chunks[4].split(':')[1])
                frame = int(chunks[5].split(':')[1])
                base['interfaces'][iface_name]['rx packets'] = {'packets': packets,
                                                           'errors': errors,
                                                           'dropped': dropped,
                                                           'overruns': overruns,
                                                           'frame' : frame}
                continue

            elif 'TX' == chunks[0] and 'packets' in chunks[1]:
                packets = int(chunks[1].split(':')[1])
                errors = int(chunks[2].split(':')[1])
                dropped = int(chunks[3].split(':')[1])
                overruns = int(chunks[4].split(':')[1])
                carrier = int(chunks[5].split(':')[1])
                base['interfaces'][iface_name]['tx packets'] = {'packets': packets,
                                                           'errors': errors,
                                                           'dropped': dropped,
                                                           'overruns': overruns,
                                                           'frame' : frame}
                continue

            elif 'collisions' in chunks[0]:
                collisions = int(chunks[0].split(':')[1])
                txqueuelen = int(chunks[1].split(':')[1])
                base['interfaces'][iface_name]['collisions'] = collisions
                base['interfaces'][iface_name]['txqueuelen'] = txqueuelen
                continue

            elif 'RX' ==  chunks[0] and 'bytes' in chunks[1]:
                r_bytes = int(chunks[1].split(':')[1])
                t_bytes = int(chunks[5].split(':')[1])
                base['interfaces'][iface_name]['rx bytes'] = r_bytes
                base['interfaces'][iface_name]['tx bytes'] = t_bytes
                continue
        else:
            # empty line between interfaces
            continue
    return base


def memory_to_dict(output):
    """Parse the output from the command `free -m` into a dictionary

    :Returns: Dictionary

    :param output: **Required** The pile of stuff outputted by running `free -m`
    :type output: String
    """
    lines = output.split('\n')
    the_keys = lines[0].split()
    memory_values = [int(x) for x in lines[1].split() if not x.startswith('M')]
    swap_values = [int(x) for x in lines[3].split() if not x.startswith('S')]

    memory = dict(zip(the_keys, memory_values))
    swap = dict(zip(the_keys, swap_values))
    return {'memory' : {'ram': memory, 'swap': swap}}


def df_to_dict(output):
    """Parse the output from the command `df` into a dictionary

    :Returns: Dictionary

    :param output: **Required** The pile of stuff outputted by running `df`
    :type output: String
    """
    response = {'filesystems': {}}
    lines = output.split('\n')
    lines.pop(0) # pop the header
    lines.pop(-1) # remove empty string from end; result of spliting on lines...
    for idx, line in enumerate(lines):
        data = line.split()
        filesystem = data[0]
        oneK_blocks = data[1]
        used = data[2]
        available = data[3]
        used_percent = data[4]
        mounted_on = data[5]
        response['filesystems'][filesystem] = {
            '1K-blocks' : oneK_blocks,
            'used' : used,
            'available' : available,
            'used percent' : used_percent,
            'mounted on' : mounted_on
            }
    return response



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
from setuptools import setup, find_packages

with open('LICENSE.txt') as the_file:
    license_info = the_file.read()

setup(name="iiqtools",
      author="Nicholas Willhite,",
      author_email="willhite.nicholas@gmail.com",
      url='https://github.com/willnx/iiqtools',
      version='1.0.0',
      packages=find_packages(),
      include_package_data=True,
      scripts=['scripts/iiq_gather_info',
               'scripts/iiq_tar_to_zip',
               'scripts/iiq_version',
               'scripts/iiq_patch'],
      license=license_info,
      description="Scripts to help debug and support InsightIQ",
      classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: System Administrators',
        'Topic :: System :: Systems Administration',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
      ]
     )



# -*- UTF-8 -*-
"""
Unit tests for the iiqtools.utils.generic module
"""
import os
import argparse
import unittest
from mock import patch

from iiqtools.utils import generic


class TestGenericUtils(unittest.TestCase):
    """A suite of test cases for the iiqtools.utils.generic module"""

    @patch.object(generic, 'stderr')
    def test_printerr_happy_path(self, mocked_stderr):
        """printerr writes to stderr"""
        generic.printerr('some error')
        self.assertEqual(mocked_stderr.write.call_count, 1)
        self.assertEqual(mocked_stderr.flush.call_count, 1)

    @patch.object(generic, 'stderr')
    def test_printerr_newline(self, mocked_stderr):
        """printerr appends a newline character automatically"""
        generic.printerr('some error')
        args, _ = mocked_stderr.write.call_args
        message = args[0]
        self.assertTrue(message.endswith('\n'))


class TestCheckPath(unittest.TestCase):
    """A suite of tests for the iiqtools.generic.check_path function"""

    def setUp(self):
        """Runs before every test case"""
        self.junk_file = '/tmp/asdfoowersdfuixlwles'

    def tearDown(self):
        """Runs after every test case"""
        try:
            os.remove(self.junk_file)
        except OSError as doh:
            if doh.errno == 2:
                # No such File, ignore
                pass
            else:
                raise

    def test_invalid_path(self):
        """Supplying an invalid file system path raises argparse.ArgumentTypeError"""
        self.assertRaises(argparse.ArgumentTypeError, generic.check_path, 'foo')

    def test_supply_directory(self):
        """Supplying an actual directory returns that value"""
        supplied_value = '/tmp'
        returned_value = generic.check_path(supplied_value)

        self.assertEqual(supplied_value, returned_value)

    def test_supply_file(self):
        """Supplying a file (not a directory) raises argparse.ArgumentTypeError"""
        f = open(self.junk_file, 'w')
        f.close()
        self.assertRaises(argparse.ArgumentTypeError, generic.check_path, self.junk_file)

if __name__ == '__main__':
    unittest.main()



# -*- coding: UTF-8 -*-
"""
Unit tests for the iiqtools.utils.database module
"""
import types
import unittest
from mock import MagicMock, patch

import iiqtools.utils.database as database


class TestDatabase(unittest.TestCase):
    """A suite of tests for the iiqtools.utils.database module"""

    def setUp(self):
        """Runs before every test case"""
        # mock away the psycopg2 module
        self.patcher = patch('iiqtools.utils.database.psycopg2')
        self.mocked_psycopg2 = self.patcher.start()
        self.mocked_connection = MagicMock()
        self.mocked_cursor = MagicMock()
        self.mocked_psycopg2.connect.return_value = self.mocked_connection
        self.mocked_connection.cursor.return_value = self.mocked_cursor
        # General mocked response
        self.mocked_cursor.fetchone.side_effect = [('foo', 'string'), ('bar', 'string'), StopIteration('test')]

    def tearDown(self):
        """Runs after every test case"""
        self.patcher.stop()

    def test_init(self):
        """Simple test that we can instantiate Database class for testing"""
        db = database.Database()
        self.assertTrue(isinstance(db, database.Database))
        self.assertTrue(db._connection is self.mocked_connection)
        self.assertTrue(db._cursor is self.mocked_cursor)

    def test_context_manager(self):
        """Database support use of `with` statement and auto-closes connection"""
        with database.Database() as db:
            pass
        self.assertTrue(self.mocked_connection.close.call_count is 1)

    def test_close(self):
        """Calling Database.close() closes the connection to the DB"""
        db = database.Database()
        db.close()
        self.assertTrue(self.mocked_connection.close.call_count is 1)

    def test_isolation_level_getter(self):
        """Property Database.isolation_level returns connection's isolation level"""
        self.mocked_connection.isolation_level = 1
        db = database.Database()
        isolation_level = db.isolation_level

        self.assertTrue(isolation_level is 1)

    def test_isolation_level_setter(self):
        """Property Database.isolation_level can set value"""
        db = database.Database()
        db.isolation_level = 7

        args, _ = self.mocked_connection.set_isolation_level.call_args
        sent_level = args[0]
        expected = 7

        self.assertEqual(sent_level, expected)

    def test_executemany(self):
        """Happy path test for the Database.executemany method"""
        db = database.Database()
        result = db.executemany(sql="SELECT * from FOO WHERE bar = %s", params=('bat',))
        self.assertTrue(isinstance(result, types.GeneratorType))

    def test_primary_key(self):
        """Method `primary_key` returns a tuple of database.Column objects"""
        db = database.Database()
        primary_key = db.primary_key(table='sometable')
        expected = (database.Column('foo', 'string'), database.Column('bar', 'string'))

        self.assertEqual(primary_key, expected)

    def test_table_schema(self):
        """Method `table_schema` returns a tuple of database.Column objects"""
        db = database.Database()
        schema = db.table_schema(table='sometable')
        expected = (database.Column('foo', 'string'), database.Column('bar', 'string'))

        self.assertEqual(schema, expected)

    def test_cluster_databases(self):
        """Method `cluster_databases` returns a list of the databases"""
        db = database.Database()
        cluster_dbs = db.cluster_databases()
        expected = ['foo', 'bar']

        self.assertEqual(cluster_dbs, expected)

    def test_tables(self):
        """Method `tables` returns a list of tables in the databases"""
        db = database.Database()
        tables = db.tables()
        expected = ['foo', 'bar']

        self.assertEqual(tables, expected)


if __name__ == '__main__':
    unittest.main()
